/*

  Autor(s)      : Jordi Ferrer Plana
  e-mail        : jferrerp@eia.udg.es
  Especialitat  : Computadors

  Project       : Practiques de computadors
  Homepage      : http://eia.udg.es/~jferrerp

  Modul         : Implementacio d'una Cua Estatica

  Fitxer        : tcua.c
  Data          : 27/03/2003 - 28/03/2003

  Compilador    : Micro Series 8051 C-Compiler V4.23C/DXT (IAR Systems 1992)
  Llibreries    :

  Notes         : -

 ----------------------------------------------------------------------------

     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version. See COPYING for more information.

 ----------------------------------------------------------------------------
*/

#include <stdio.h>  /* printf */

#include "tcua.h"   /* La definicio */

/*!
   MACROS: S'implementa aixi per que no hi hagi massa enniuament de funcions.
*/
#define TCUA_PLENA  Cua->Punter_Fi == Cua->Punter_Inici && Cua->Buida == 0
#define TCUA_BUIDA  Cua->Punter_Fi == Cua->Punter_Inici && Cua->Buida != 0

/*!
   MACRO de Control d'errors del modul: Envia per port serie l'error
	                                     utilitzant el "printf".
*/
#define TCUA_ERROR_INTERN(Codi_Error,Rutina,Descripcio)                    \
   printf ( "\n" );                                                        \
   printf ( "- CODI D'ERROR : %d\n", ##Codi_Error ); /* Escriure pistes */ \
   printf ( "  MODUL        : TCUA\n" );             /* per localitzar */  \
   printf ( "  FITXER       : %s\n", __FILE__ );     /* l'error */         \
   printf ( "  LINIA        : %d\n", __LINE__ );                           \
   printf ( "  FUNCIO       : %s\n", ##Rutina );                           \
   printf ( "  DESCRIPCIO   : %s\n", ##Descripcio );                       \
                                                                           \
   exit ( ##Codi_Error );               /* Plegar amb el Numero d'Error */ \

/*!
   Funcio per inicialitzar la cua.
*/
void TCUA_Crear ( TCua *Cua )
{
   Cua->Punter_Inici = Cua->Punter_Fi = 0;
   Cua->Buida = 1;
}

/*!
   Retorna 1 si la cua es buida, 0 en cas contrari.
*/
bit TCUA_Buida ( TCua *Cua )
{
   return TCUA_BUIDA;
}

/*!
   Retorna 1 si la cua es plena, 0 en cas contrari.
*/
bit TCUA_Plena ( TCua *Cua )
{
   return TCUA_PLENA;
}

/*!
   En cas que la cua no estigui plena, col.loca la dada entrada per parametre
   a la cua. Si la cua es plena, genera un error i acaba l'execucio.
*/
void TCUA_Encuar ( TCua *Cua, TCUA_Element Dada )
{
   if ( TCUA_PLENA )
   {
      TCUA_ERROR_INTERN ( 1, "void TCUA_Encuar ( TCua *Cua, Element_Cua Dada );",
                             "Intent d'encuar amb la cua plena!" );
   }
   else
   {
      Cua->Dada[Cua->Punter_Fi] = Dada;
      Cua->Punter_Fi++;
      Cua->Punter_Fi %= TCUA_MAX_ELEMENTS;
      Cua->Buida = 0;
   }
}

/*!
   En cas que la cua no estigui buida, elimina la primera dada de la cua.
   Si la cua es buida, genera un error i acaba l'execucio.
*/
void TCUA_Desencuar ( TCua *Cua )
{
   if ( TCUA_BUIDA )
   {
      TCUA_ERROR_INTERN ( 2, "void TCUA_Desencuar ( TCua *Cua );",
                             "Intent de desencuar amb la cua buida!" );
   }
   else
   {
      Cua->Punter_Inici++;
      Cua->Punter_Inici %= TCUA_MAX_ELEMENTS;
      Cua->Buida = 1;
   }
}

/*!
   En cas que la cua no estigui buida, retorna la primera dada de la cua.
   Si la cua es buida, genera un error i acaba l'execucio.
*/
TCUA_Element TCUA_Primer ( TCua *Cua )
{
   if ( TCUA_BUIDA )
   {
      TCUA_ERROR_INTERN ( 3, "TCUA_Element TCUA_Primer ( TCua *Cua );",
                             "Intent d'obtenir un element amb la cua buida!" );
   }

   return Cua->Dada[Cua->Punter_Inici];
}
